@@ -1,3 +1,24 @@
+c 
+c     Authors:
+c     Reinhard Furrer
+c       
+c  Copyright (C) 2017 -- 2017 Reinhard Furrer
+c 
+c This program is free software; you can redistribute it and/or
+c modify it under the terms of the GNU General Public License
+c as published by the Free Software Foundation; either version 3
+c of the License, or (at your option) any later version.
+c 
+c This program is distributed in the hope that it will be useful,
+c but WITHOUT ANY WARRANTY; without even the implied warranty of
+c MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+c GNU General Public License for more details.
+c 
+c You should have received a copy of the GNU General Public License
+c along with this program; if not, write to the Free Software
+c Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.  
+
+
       subroutine amuxmat (n,m,p, x, y, a,ja,ia)
       implicit none
       integer n, m, p, ja(*), ia(*)
@@ -1869,7 +1890,8 @@ c-----------
 c       x = The solution of  R x = b .
 c--------------------------------------------------------------------
 c     Reinhard Furrer June 2008, April 2012
-
+      
+      k = 0
       if (r(ir(n+1)-1) .eq. 0.0 ) goto 5
       do l=1,p
          x(n,l) = b(n,l) / r(ir(n+1)-1)
